#!/usr/local/bin/perl -w
# tkbutton.pl -- demo of button widgets

use strict;
use Tk;

# declaration of the used variables
my ($mw, $btn1, $btn2, $btn3, $btn4, $btnxit);
my ($btnin, $btnvar, $btndis);
my ($now, $then);

# first of all, create a main window
$mw = MainWindow->new();

# create some buttons

#### calling an anonymous subroutine
$btn1 = $mw->Button(-text    => "Anonymous Subroutine",
                    -command => sub { system("date"); });

#### calling a named subroutine
$btn2 = $mw->Button(-text    => "Calling a named sub",
                    -command => \&hello);

#### calling a named subroutine with parameters passing
$btn3 = $mw->Button(-text    => "Passing two args",
                    -command => [\&btn, "Arg1", "Arg2"]);

#### calling the same named subroutine, with
#### another parameter
$btn4 = $mw->Button(-text    => "Passing one arg",
                    -command => [\&btn, "XYZ"]);

#### calling an anonymous subroutine,
#### quitting the program
$btnxit = $mw->Button(-text    => "Exit",
                      -command => sub { exit 0; });

#### creating a button with init variable
$now    = localtime(time);
$btnin  = $mw->Button(-text    => $now,
                      -command => [\&btn, "time", "init"]);

#### creating a button with variable text
$then   = localtime(time);
$btnvar = $mw->Button(-textvariable => \$then,
                      -command => sub { $then = localtime(time) });

#### creating a disabled button
$btndis = $mw->Button(-text    => "I am disabled",
                      -state   => "disabled",
                      -command => sub { print "never called" });

# display everything
$btn1->pack(-fill => 'both', -expand => 'yes');
$btn2->pack(-fill => 'both', -expand => 'yes');
$btn3->pack(-fill => 'both', -expand => 'yes');
$btn4->pack(-fill => 'both', -expand => 'yes');
$btndis->pack(-fill => 'both', -expand => 'yes');
$btnin->pack(-fill => 'both', -expand => 'yes');
$btnvar->pack(-fill => 'both', -expand => 'yes');
$btnxit->pack(-fill => 'both', -expand => 'yes');

# enter main event collecting loop
MainLoop;

# helper routines
sub hello { print "Hello World!\n"; }
sub btn
{
    my (@args) = @_;
    my $arg;

    print "Button pressed.\n";
    foreach $arg (@args) {
        print "\tArg: $arg\n";
    }
}
