#!/usr/local/bin/perl -w
# tktext.pl -- demo of the Tk's Text widget
# (C) 1997/01/29 Farid Hajji

use strict;
use Tk;

# declaration of our variables
my ($mw, $frame);
my ($text, $yscr, $xscr);
my ($fbut, $bread, $bask, $bxit);

# first create a main window
# and a frame, where to put the text widget
#     together with the scrollbars
$mw    = MainWindow->new();
$frame = $mw->Frame();

# now create a Text Widget,
# and two scrollbars in the Frame

# create the text widget
$text = $frame->Text(-wrap => 'none');

# create two scrollbars,
# and bind them to the $text widget
$yscr = $frame->Scrollbar(-command =>
                             [yview => $text],
                          -orient  => 'vertical');
$xscr = $frame->Scrollbar(-command =>
                             [xview => $text],
                          -orient  => 'horizontal');

# bind the $text widget to the scrollbars
$text->configure(-yscrollcommand => [ set => $yscr ]);
$text->configure(-xscrollcommand => [ set => $xscr ]);

# now, pack everything

# --- the frame
$frame->pack(-expand => 'yes', -fill => 'both');

# --- the scrollbars
$yscr->pack(-side => 'right',  -fill => 'y');
$xscr->pack(-side => 'bottom', -fill => 'x');

# --- the text widget
$text->pack(-expand => 'yes', -fill => 'both',
            -side => 'left');

# ---------------------------------
# Add some Buttons in another frame
$fbut = $mw->Frame();
$bread = $fbut->Button(-text    => 'Read Source',
                       -command => \&do_read);
$bask  = $fbut->Button(-text    => 'Upper me!',
                       -command => \&do_upper);
$bxit  = $fbut->Button(-text    => 'Exit',
                       -command => sub { exit 0 });
$fbut->pack(-side => 'bottom', -expand => 'both',
            -fill => 'both');
$bread->pack(-side => 'left');
$bask->pack(-side => 'left');
$bxit->pack(-side => 'left');


# now, enter the main event loop
MainLoop;

# ------------ helper routines -------------
sub do_read {
    # read in the source into text widget
    # the source is in $0
    open (IFILE, "< $0") or die "can't open $0: $!\n";
    while (<IFILE>) {
        # DON'T chop()
        $text->insert("end", $_);
    }
    close (IFILE);
}

sub do_upper {
    # upper-case every char of every line
    my $alltext = $text->get("1.0", "end");
    $alltext =~ s/(.*?)\n/uc($1) . "\n"/meg;
    $text->delete("1.0", "end");
    $text->insert("end", $alltext);
}
