#!/usr/local/bin/perl -w
# mywc.pl -- wc.lex simuliert mit Parse::Lex

use Parse::Lex;                    # CPAN-Modul

use Getopt::Std;
use vars qw ($opt_v);
getopts("v") || die "Usage: $0 [-v] [inputfile]\n";

# Die Lex-Spezifikation liegt im folgenden Format vor:
#     Token-Symbol, regulaerer Ausdruck [, Aktionssubroutine]
#     Wobei die Aktionssubroutine als ersten Parameter das Token-Symbol
#                    und als zweiten Parameter den Wert des Tokens
#                    bekommt. Der Rueckgabewert ist der neue Tokenwert.
my @token = (
	     'WORD'  => '[^ \t\n]+'
	             => sub { my ($tok,$tokval) = @_;
			      ++$words; $chars += length($tokval);
			      return $tokval; },
	     'EOL'   => '\n'
	             => sub { ++$lines; ++$chars;
			      return $_[1]; },
	     'OTHER' => '.'
	             => sub { ++$chars; return $_[1]; }
	    );

# Lexer-Objekt instanziieren:
my $lexer = Parse::Lex->new(@token);

# Eingabe entweder von STDIN oder einer Eingabedatei:
if (defined $ARGV[0]) {
    open(MYDATA, $ARGV[0]) or die "$!\n";
    $lexer->from(\*MYDATA);
} else {
    # Redundant, da Default: $lexer->from(\*STDIN);
    # Alternativ: $lexer->from($inputstring);
}

# Nun holen wir Token nach Token herein...
# Wenn die Eingabe komplett gelesen wurde -> $Parse::Token::EOI
while (($token = $lexer->next()) ne $Parse::Token::EOI) {
    print $token->name(), ": >", $token->text(), "<\n"
	if defined $opt_v;
}

# Wir koennen nun auf unsere Zaehlervariablen zugreifen:
print "Lines: $lines; Words: $words; Chars: $chars\n";
